-----------------------------------------------------------------------------------------------
-- Client Lua Script for MouseLockUtil by Xan the Dragon // Gyro Egan
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
require "Window"
-----------------------------------------------------------------------------------------------
-- MouseLockUtil Module Definition
-----------------------------------------------------------------------------------------------
local MouseLockUtil = {}

--The last state of mouse lock
local LastState = false

--The last sprite selection. This also functions as the default for startup in the event that there is no data.
local LastIndex = 2

--Variable storing if the mod is ready to go, a variable storing whether or not the value was updated after we marked as ready, and a value storing if the restore function was even called.
local Ready = false
local DidInitialUpdate = false
local RestoreFunctionCalled = false

--The sprites offered
local SpriteData = {
	{
		Sprite = "";
		Size = 1;
		Name = "Off";
	};
	{
		Sprite = "CRB_MegamapSprites:sprMap_PlayerDot"; --Sprite filename
		Size = 8; --The size in pixels of this image.
		Name = "Dot"; --Display name
	};
	{
		Sprite = "Crafting_CoordSprites:sprCoord_AdditiveTargetGreenSmall";
		Size = 30; 
		Name = "Green Crosshair";
	};
	{
		Sprite = "Crafting_CoordSprites:sprCoord_AdditiveTargetRedSmall";
		Size = 30;
		Name = "Red Crosshair";
	};
	--Feel free to add more.
}


--Initialization core function
function MouseLockUtil:new(Object)
	Object = Object or {}
	setmetatable(Object, self)
	self.__index = self
		
	return Object
end

--Apollo initialization function
function MouseLockUtil:Init()
	Apollo.RegisterAddon(self, true, "Set Mouselock Cursor", {})
end

--Data persistence saving
function MouseLockUtil:OnSave(DataLevel)
	if DataLevel == GameLib.CodeEnumAddonSaveLevel.Character then
		return {LastIndex} --This necessity irritates me.
	end
end

--Data persistence loading
function MouseLockUtil:OnRestore(DataLevel, LoadedData)
	RestoreFunctionCalled = true
	if DataLevel == GameLib.CodeEnumAddonSaveLevel.Character then
		LastIndex = LoadedData[1] or LastIndex --LastIndex has a default value, so it will just be setting to itself if there's nothing to load. Is this even necessary?
		Ready = true
	end
end

--Addon initialization function
function MouseLockUtil:OnLoad()
	self.PrimaryWindowXML = XmlDoc.CreateFromFile("MouseLockUtil.xml")
	self.PrimaryWindowXML:RegisterCallback("OnDocLoaded", self)
	Apollo.RegisterTimerHandler("UpdateTimer", "CheckForUpdate", self)
	Apollo.RegisterTimerHandler("Loader", "LoadTimeout", self)
end

--Document loading complete callback
function MouseLockUtil:OnDocLoaded()
	if self.PrimaryWindowXML ~= nil and self.PrimaryWindowXML:IsLoaded() then
		self.Crosshairs = Apollo.LoadForm(self.PrimaryWindowXML, "Cursor", nil, self)
		self.Config = Apollo.LoadForm(self.PrimaryWindowXML, "Cfg", self)
		if self.Crosshairs == nil or self.Config == nil or self.Config:FindChild("SliderBar") == nil or self.Config:FindChild("EditBox") == nil then
			Apollo.AddAddonErrorText(self, "The UI failed to load for an unknown reason. Try running /reloadui.")
			Apollo.AddAddonErrorText(self, "Debug. Which UI elements are nil? Crosshairs, Config, Slider, Edit = " 
				.. tostring(self.Crosshairs == nil) .. ", "
				.. tostring(self.Config == nil) .. ", "
				.. tostring(self.Config:FindChild("SliderBar") == nil) .. ", " 
				.. tostring(self.Config:FindChild("EditBox") == nil)
			)
			return
		end
		self.Slider = self.Config:FindChild("SliderBar")
		self.EditBox = self.Config:FindChild("EditBox")
		self.Slider:Show(true, true)
		self.EditBox:Show(true, true)
		self.Config:Show(false, true)
		
		local Active = GameLib.IsMouseLockOn()
		self.Crosshairs:Show(Active, true)
		LastState = Active
		
		MouseLockUtil.Crosshairs = self.Crosshairs
		MouseLockUtil.Slider = self.Slider
		MouseLockUtil.EditBox = self.EditBox
		
		self.Slider:SetMinMax(1, #SpriteData, 1)
		
		Apollo.CreateTimer("UpdateTimer", 0.125, true)
		Apollo.CreateTimer("Loader", 0.2, false)
	end
end

--Config function.
function MouseLockUtil:OnConfigure()
	self.Config:Show(true, true)
end

--Load check function.
function MouseLockUtil:LoadTimeout()
	if not RestoreFunctionCalled then
		Ready = true
	end
end

--Primary timer function
function MouseLockUtil:CheckForUpdate()
	if not Ready then return end
	if not DidInitialUpdate then
		MouseLockUtil:CheckForNewSelection(LastIndex) --Calling with an arg will forcefully set the value no matter what.
		DidInitialUpdate = true
	end
	MouseLockUtil:CheckForNewSelection() --Check for a changed option. It's safe to call this again if the update was done above, since this will do nothing.
	if LastState == GameLib.IsMouseLockOn() then return end
	local Active = GameLib.IsMouseLockOn()
	self.Crosshairs:Show(Active, true)
	LastState = Active
end

--Set cursor.
function MouseLockUtil:Set(Index)
	--No ready check. This is called by CheckForNewSelection
	local Data = SpriteData[Index]
	local HalfSize = Data.Size / 2
	MouseLockUtil.EditBox:SetText("Cursor: " .. Data.Name)
	MouseLockUtil.Crosshairs:SetSprite(Data.Sprite)
	MouseLockUtil.Crosshairs:SetAnchorOffsets(-HalfSize, -HalfSize, HalfSize, HalfSize)
end

---------------------------------------------------------------------------------------------------
-- Cfg Functions
---------------------------------------------------------------------------------------------------

function MouseLockUtil:CheckForNewSelection(ForcedValue)
	--No ready check. This is called by CheckForUpdate
	local CurrentValue = ForcedValue or MouseLockUtil.Slider:GetValue()
	if ForcedValue == nil and CurrentValue == LastIndex then 
		return
	end
	if ForcedValue then 
		MouseLockUtil.Slider:SetValue(ForcedValue)
	end
	MouseLockUtil:Set(CurrentValue)
	LastIndex = CurrentValue
end

--Instantiate this addon
local MouseLockUtilInst = MouseLockUtil:new()
MouseLockUtilInst:Init()